package com.cuit.solarenergy.controller;

import com.cuit.solarenergy.base.annotation.AopLog;
import com.cuit.solarenergy.base.annotation.AuthToken;
import com.cuit.solarenergy.entity.vo.ret.RetResponse;
import com.cuit.solarenergy.entity.vo.ret.RetResult;
import com.cuit.solarenergy.exception.ParamsException;
import com.cuit.solarenergy.service.DataImportService;
import com.cuit.solarenergy.utils.ErrorMessage;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-11  14:49
 * @Description: 数据导入
 * @Version: 1.0
 */
@Api(tags = "10、数据导入")
@ApiSupport(order = 10)
@Slf4j
@RestController()
public class DataImportController {

    @Autowired
    private DataImportService dataImportService;

    /**
     * @author: hpj
     * @date: 2022/5/17 18:34
     * @version: 1.0
     */
    @AopLog
    @AuthToken
    @ApiOperation("1、以excel（xlsx）文件的形式导入站点信息")
    @PreAuthorize("hasAuthority('dataimport:upLoadRailwayDataByExcel')")
    @RequestMapping(value = "/dataimport/upLoadRailwayDataByExcel", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<String>> upLoadRailwayDataByExcel(@RequestPart(value = "file") MultipartFile railwayExcelFile) {
        try {
            List<String> retMsg = dataImportService.upLoadRailwayDataByExcel(railwayExcelFile);
            return RetResponse.makeOKRsp(retMsg);
        } catch (ParamsException p) {
            p.printStackTrace();
            return RetResponse.makeErrRsp("文件错误！" + p.getMessage());
        } catch (ErrorMessage e) {
            dataImportService.restoreSequence(1);
            e.printStackTrace();
            return RetResponse.makeErrRsp("导入数据出错！" + e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常......" + e.getMessage());
        }
    }


    /**
     * 下载Excel模板
     *
     * @return
     */
    @AopLog
    @AuthToken
    @ApiOperation("2、下载Excel模板")
    @PreAuthorize("hasAuthority('excel:template')")
    @RequestMapping(value = "/excel/template", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public void downloadTemplate(HttpServletResponse response) throws Exception {

//        response.setCharacterEncoding("UTF-8");
//        response.setContentType("application/vnd.ms-excel");
//
//        String fileName = URLEncoder.encode("导入信息模板","UTF-8");
//        response.setHeader("Content-disposition","attachment;filename=" + fileName + ".xlsx");
//
//        ServletOutputStream outputStream = response.getOutputStream();
//
//        InputStream exportTemplate = Application.class.getClassLoader().getResourceAsStream("export-template.xlsx");
//
//        EasyExcel.write(outputStream)
//                .withTemplate(exportTemplate)
//                .sheet()
//                .doWrite((Collection<?>) null);


        dataImportService.export(response);

    }


     @AopLog
    @AuthToken
    @ApiOperation("3、上传卫星数据及无人机数据压缩包")
    @PreAuthorize("hasAuthority('dataimport:upLoadStaUAVTifData')")
    @RequestMapping(value = "/dataimport/upLoadStaUAVTifData", method = {RequestMethod.POST})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "railwayStaUAVTifData", value = "选择多个文件或Zip压缩包", required = true),
       })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
//    public RetResult<List<String>> upLoadStaUAVTifData(@RequestPart(value = "file") MultipartFile railwayStaUAVTifData) {
    public RetResult<List<String>> upLoadStaUAVTifData(@RequestParam("railwayStaUAVTifData") MultipartFile[] railwayStaUAVTifData) {
        try {
//            List<String> retMsg = dataImportService.upLoadRailwayStaUAVTifData(railwayStaUAVTifData);
            List<String> retMsg = dataImportService.upLoadRailwayStaUAVTifData(railwayStaUAVTifData);
            return RetResponse.makeOKRsp(retMsg);
        } catch (ParamsException p) {
            p.printStackTrace();
            return RetResponse.makeErrRsp("文件错误！" + p.getMessage());
        } catch (ErrorMessage e) {
            dataImportService.restoreSequence(1);
            e.printStackTrace();
            return RetResponse.makeErrRsp("导入数据出错！" + e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常......" + e.getMessage());
        }
    }

    @AopLog
    @AuthToken
    @ApiOperation("4、删除已导入的卫星及无人机数据")
    @PreAuthorize("hasAuthority('dataimport:deleteGeoserverStaUAVTifData')")
    @RequestMapping(value = "/dataimport/deleteGeoserverStaUAVTifData", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<String> deleteGeoserverStaUAVTifData() {
        try {
            String retMsg = dataImportService.deleteGeoserverStaUAVTifData();
            return RetResponse.makeOKRsp(retMsg);
        } catch (ParamsException p) {
            p.printStackTrace();
            return RetResponse.makeErrRsp("文件错误！" + p.getMessage());
        } catch (ErrorMessage e) {
            dataImportService.restoreSequence(1);
            e.printStackTrace();
            return RetResponse.makeErrRsp("导入数据出错！" + e.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常......" + e.getMessage());
        }
    }

}


