package com.cuit.solarenergy.controller;

import com.cuit.solarenergy.base.annotation.AopLog;
import com.cuit.solarenergy.base.annotation.AuthToken;
import com.cuit.solarenergy.entity.TMinio;
import com.cuit.solarenergy.entity.TSiteBaseInfo;
import com.cuit.solarenergy.entity.TStreamBaseInfo;
import com.cuit.solarenergy.entity.vo.req.*;
import com.cuit.solarenergy.entity.vo.ret.*;
import com.cuit.solarenergy.exception.ParamsException;
import com.cuit.solarenergy.minio.MinioService;
import com.cuit.solarenergy.service.DataManagerService;
import com.cuit.solarenergy.service.TSiteBaseInfoService;
import com.cuit.solarenergy.service.TStreamBaseInfoService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-06  10:24
 * @Description: 数据管理模块
 * @Version: 1.0
 */
@Api(tags = "6、数据管理模块")
@ApiSupport(order = 6)
@Slf4j
@RestController()
@Transactional(rollbackFor = Exception.class)
public class DataManagerController {

    @Resource
    private DataManagerService dataManagerService;

    @Resource
    private TStreamBaseInfoService tStreamBaseInfoService;

    @Resource
    private TSiteBaseInfoService tSiteBaseInfoService;

    @Resource
    private MinioService minioService;

    /**
     * @Description: 数据管理模块获取数据查询列表接口
     * @Param: dataManageGetVoReq, query
     * @return: dataPage
     * @Author: LuoSir
     * @Date: 2023/7/11
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 1)
    @ApiOperation("1、数据管理模块获取数据查询列表接口")
    @PreAuthorize("hasAuthority('data:manager:getList')")
    @RequestMapping(value = "/data/manager/getList", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<DataManageGetVoRet>> getList(@RequestBody DataManageGetVoReq req) {
        PageVo<DataManageGetVoRet> dataPage;
        try {
            dataPage = dataManagerService.getList(req);
            return RetResponse.makeOKRsp(dataPage);
        } catch (RuntimeException runtimeException) {
            return RetResponse.makeErrRsp(runtimeException.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 6)
    @ApiOperation("6、流域分页查询")
    @PreAuthorize("hasAuthority('data:manager:getStreamList')")
    @RequestMapping(value = "/data/manager/getStreamList", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<TStreamBaseInfo>> getStreamList(@RequestBody StreamListGetVoReq req) {
        PageVo<TStreamBaseInfo> dataPage;
        try {
            dataPage = tStreamBaseInfoService.getList(req);
            return RetResponse.makeOKRsp(dataPage);
        } catch (RuntimeException runtimeException) {
            return RetResponse.makeErrRsp(runtimeException.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 7)
    @ApiOperation("7、站点分页查询")
    @PreAuthorize("hasAuthority('data:manager:getSiteList')")
    @RequestMapping(value = "/data/manager/getSiteList", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<TSiteBaseInfo>> getSiteList(@RequestBody SiteListGetVoReq req) {
        PageVo<TSiteBaseInfo> dataPage;
        try {
            dataPage = tSiteBaseInfoService.getSiteList(req);
            return RetResponse.makeOKRsp(dataPage);
        } catch (RuntimeException runtimeException) {
            return RetResponse.makeErrRsp(runtimeException.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 8)
    @ApiOperation("8、图片和视频分页查询")
    @PreAuthorize("hasAuthority('data:manager:getPicAviList')")
    @RequestMapping(value = "/data/manager/getPicAviList", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<PageVo<TMinio>> getPicAviList(@RequestBody PicAviListGetVoReq req) {
        PageVo<TMinio> dataPage;
        try {
            dataPage = minioService.getPicAviList(req);
            return RetResponse.makeOKRsp(dataPage);
        } catch (RuntimeException runtimeException) {
            return RetResponse.makeErrRsp(runtimeException.getMessage());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }



    /**
     * 方法功能：数据管理模块新增数据接口(这个需要Excel导入，暂不处理)
     *
     * @param dataManageAddVoReq
     * @return
     * @date 2023/7/6 10:45
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 2)
    @ApiOperation("2、数据管理模块新增数据接口")
    @PreAuthorize("hasAuthority('data:manager:addData')")
    @RequestMapping(value = "/data/manager/addData", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> addData(DataManageAddVoReq dataManageAddVoReq) {
        return null;
    }

    /**
     * 方法功能：数据管理模块编辑数据接口
     *
     * @param dataManageUpdateVoReqList
     * @return
     * @date 2023/7/6 10:51
     * @author LLW
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 3)
    @ApiOperation("3、数据管理模块编辑数据接口")
    @PreAuthorize("hasAuthority('data:manager:updateData')")
    @RequestMapping(value = "/data/manager/updateData", method = {RequestMethod.PUT})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> updateData(@RequestBody List<DataManageUpdateVoReq> dataManageUpdateVoReqList) {
        Boolean flag = false;
        try {
            flag = dataManagerService.updateData(dataManageUpdateVoReqList);
            return RetResponse.makeOKRsp(flag);
        } catch (ParamsException paramsException) {
            return RetResponse.makeErrRsp(paramsException.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常......");
        }

    }

    /**
     * 方法功能：数据管理模块删除数据接口
     *
     * @param ids
     * @return
     * @date 2023/7/6 10:55
     * @author Tjb
     */
    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 4)
    @ApiOperation("4、数据管理模块删除数据接口")
    @PreAuthorize("hasAuthority('data:manager:deleteData')")
    @RequestMapping(value = "/data/manager/deleteData", method = {RequestMethod.DELETE})
    @ApiImplicitParam(name = "ids", value = "要删除的ids串", required = true, dataTypeClass = String.class, paramType = "query")
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<Boolean> deleteData(String ids) {
        Boolean flag = false;
        try {
            flag = dataManagerService.deleteData(ids);
            return RetResponse.makeOKRsp(flag);
        } catch (ParamsException paramsException) {
            return RetResponse.makeErrRsp(paramsException.getMsg());
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常......");
        }

    }

    /**
     * @Description: 根据流域获取所有站点名
     * @Param:
     * @return: dataSiteByStreamRet
     * @Author: LuoSir
     * @Date: 2023/7/17
     */

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 5)
    @ApiOperation("5、根据流域获取所有站点名")
    @PreAuthorize("hasAuthority('data:manager:getSiteByStream')")
    @RequestMapping(value = "/data/manager/getSiteByStream", method = {RequestMethod.GET})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<DataSiteTree>> getSiteByStream() {
        List<DataSiteTree> dataSiteTreeList;
        dataSiteTreeList = dataManagerService.getSiteByStream();
        return RetResponse.makeOKRsp(dataSiteTreeList);
    }

}