package com.cuit.solarenergy.controller;

import com.cuit.solarenergy.base.annotation.AopLog;
import com.cuit.solarenergy.base.annotation.AuthToken;
import com.cuit.solarenergy.entity.vo.req.ChartBaseInfoVoReq;
import com.cuit.solarenergy.entity.vo.ret.*;
import com.cuit.solarenergy.service.TSiteBaseInfoService;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.github.xiaoymin.knife4j.annotations.ApiSupport;
import io.swagger.annotations.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.constraints.NotNull;
import java.util.List;

/**
 * @BelongsProject: railway
 * @BelongsPackage: com.cuit.railway.controller
 * @Author: Luolw
 * @Date: 2023/7/17 15:15
 * @Description:
 */

@Api(tags = "7、统计分析模块")
@ApiSupport(order = 7)
@Slf4j
@RestController()
@Transactional(rollbackFor = Exception.class)
public class StatisticAnalysisController {
    @Resource
    private TSiteBaseInfoService tSiteBaseInfoService;

    @AuthToken
    @AopLog
    @ApiOperationSupport(order = 1)
    @ApiOperation("统计分析模块根据经纬度查询最近的三个站点")
    @PreAuthorize("hasAuthority('data:analysis:getList')")
    @RequestMapping(value = "/data/analysis/getList", method = {RequestMethod.POST})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "lon", value = "站点经度", required = true, dataTypeClass = Double.class, paramType = "query"),
            @ApiImplicitParam(name = "lat", value = "站点纬度", required = true, dataTypeClass = Double.class, paramType = "query")
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<DataBaseSeparation>> getList(Double lon, Double lat) {
        List<DataBaseSeparation> dataBaseSeparationList;
        try {
            dataBaseSeparationList = tSiteBaseInfoService.getList(lon, lat);
            return RetResponse.makeOKRsp(dataBaseSeparationList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    /**
     *方法功能：返回所有站点经纬度
     *
     * @param
     * @date 2023/8/7 9:27
     * @author Tjb
     * @return
     */

    @AopLog
    @ApiOperationSupport(order = 2)
    @ApiOperation("返回所有站点经纬度")
    @PreAuthorize("hasAuthority('data:analysis:getAllSiteList')")
    @RequestMapping(value = "/data/analysis/getAllSiteList", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<SiteBaseInfoVoRet>> getAllSiteList() {
        List<SiteBaseInfoVoRet> siteBaseInfoVoRetList;
        try {
            siteBaseInfoVoRetList = tSiteBaseInfoService.getAllSiteList();
            return RetResponse.makeOKRsp(siteBaseInfoVoRetList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }


//    @AuthToken
//    @AopLog
//    @ApiOperationSupport(order = 3)
//    @ApiOperation("根据站点、水质参数获取曲线所需的列表")
//    @PreAuthorize("hasAuthority('data:analysis:queryDataCurveByParams')")
//    @RequestMapping(value = "/data/analysis/queryDataCurveByParams", method = {RequestMethod.POST})
//    @ApiResponses({
//            @ApiResponse(code = 200, message = "成功"),
//            @ApiResponse(code = 401, message = "访问未经授权"),
//            @ApiResponse(code = 403, message = "访问被禁止的"),
//            @ApiResponse(code = 404, message = "未找到访问接口"),
//            @ApiResponse(code = 500, message = "服务器内部错误"),
//            @ApiResponse(code = 900, message = "失败")
//    })
//    public RetResult<List<StatisticQueryVoRet>> queryDataCurveByParams(StatisticQueryVoReq StatisticQueryVoReq) {
//        List<StatisticQueryVoRet> statisticQueryList;
//        try {
//            statisticQueryList = tSiteBaseInfoService.queryDataCurveByParams(StatisticQueryVoReq);
//            return RetResponse.makeOKRsp(statisticQueryList);
//        } catch (ParamsException paramsException) {
//            return RetResponse.makeErrRsp(paramsException.getMessage());
//        } catch (Exception e) {
//            e.printStackTrace();
//            return RetResponse.makeErrRsp("服务器异常.....");
//        }
//    }

    /**
     *方法功能：返回绘制图表的所有信息
     *
     * @param
     * @date 2023/8/7 9:27
     * @author Tjb
     * @return
     */

    @AopLog
    @ApiOperationSupport(order = 4)
    @ApiOperation("返回绘制图表的所有信息")
    @PreAuthorize("hasAuthority('data:analysis:getAllChartList')")
    @RequestMapping(value = "/data/analysis/getAllChartList", method = {RequestMethod.POST})
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<StatisticAnalysisVoRet>> getAllChartList( @RequestBody ChartBaseInfoVoReq chartBaseInfoVoReq) {
        List<StatisticAnalysisVoRet> statisticAnalysisVoRets;
        try {
            statisticAnalysisVoRets = tSiteBaseInfoService.getAllChartList(chartBaseInfoVoReq);
            return RetResponse.makeOKRsp(statisticAnalysisVoRets);
         } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AopLog
    @ApiOperationSupport(order = 5)
    @ApiOperation("返回指定流域指定水质参数各站点数值信息")
    @PreAuthorize("hasAuthority('data:analysis:getAllChartList')")
    @RequestMapping(value = "/data/analysis/getValueOfSite", method = {RequestMethod.GET})
    @ApiImplicitParams({
            @ApiImplicitParam(name = "streamName",value = "流域名称(金沙江)",type =  "String",required = true),
            @ApiImplicitParam(name = "waterParameter",value = "水质参数名称(叶绿素)",type =  "String",required = true)
    })
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<SiteAndValueVoRet>> getValueOfSite(@NotNull @RequestParam String streamName, @NotNull @RequestParam String waterParameter) {
        List<SiteAndValueVoRet> siteAndValueVoRets;
        try {
            siteAndValueVoRets = tSiteBaseInfoService.getValueOfSite(streamName, waterParameter);
            return RetResponse.makeOKRsp(siteAndValueVoRets);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AopLog
    @ApiOperationSupport(order = 6)
    @ApiOperation("返回指定流域近三年的站点统计信息")
    @PreAuthorize("hasAuthority('data:analysis:getAllChartList')")
    @RequestMapping(value = "/data/analysis/getAllInfoForThreeYear", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "streamName",value = "流域名称(金沙江)",type =  "String",required = true)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<YearAndCountVoRet>> getAllInfoForThreeYear(@NotNull @RequestParam String streamName) {
        List<YearAndCountVoRet> yearAndCountVoRetList;
        try {
            yearAndCountVoRetList = tSiteBaseInfoService.getAllInfoForThreeYear(streamName);
            return RetResponse.makeOKRsp(yearAndCountVoRetList);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AopLog
    @ApiOperationSupport(order = 7)
    @ApiOperation("返回无人机各年月份统计信息")
    @PreAuthorize("hasAuthority('data:analysis:getAllChartList')")
    @RequestMapping(value = "/data/analysis/getCountOfUAV", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "streamName",value = "流域名称(金沙江)",type =  "String",required = true)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<MonthAndCountVoRet>> getCountOfUAV(@NotNull @RequestParam String streamName) {
        List<MonthAndCountVoRet> monthAndCountVoRets;
        try {
            monthAndCountVoRets = tSiteBaseInfoService.getCountOfUAV(streamName);
            return RetResponse.makeOKRsp(monthAndCountVoRets);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

    @AopLog
    @ApiOperationSupport(order = 8)
    @ApiOperation("返回卫星遥感各年月份统计信息")
    @PreAuthorize("hasAuthority('data:analysis:getAllChartList')")
    @RequestMapping(value = "/data/analysis/getCountOfSAT", method = {RequestMethod.GET})
    @ApiImplicitParam(name = "streamName",value = "流域名称(金沙江)",type =  "String",required = true)
    @ApiResponses({
            @ApiResponse(code = 200, message = "成功"),
            @ApiResponse(code = 401, message = "访问未经授权"),
            @ApiResponse(code = 403, message = "访问被禁止的"),
            @ApiResponse(code = 404, message = "未找到访问接口"),
            @ApiResponse(code = 500, message = "服务器内部错误"),
            @ApiResponse(code = 900, message = "失败")
    })
    public RetResult<List<MonthAndCountVoRet>> getCountOfSAT(@NotNull @RequestParam String streamName) {
        List<MonthAndCountVoRet> monthAndCountVoRets;
        try {
            monthAndCountVoRets = tSiteBaseInfoService.getCountOfSAT(streamName);
            return RetResponse.makeOKRsp(monthAndCountVoRets);
        } catch (Exception e) {
            e.printStackTrace();
            return RetResponse.makeErrRsp("服务器异常.....");
        }
    }

}
