package com.cuit.solarenergy.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.entity.MapService;
import com.cuit.solarenergy.entity.TSamplingDataDetails;
import com.cuit.solarenergy.entity.vo.req.MultiRSDetectionReq;
import com.cuit.solarenergy.entity.vo.req.MultiRSReq;
import com.cuit.solarenergy.entity.vo.req.UAVDetectionReq;
import com.cuit.solarenergy.entity.vo.ret.*;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * @author PYJ
 * @description 针对表【t_sampling_data_details(采样数据明细表)】的数据库操作Mapper
 * @createDate 2023-06-08 11:37:12
 * @Entity com.cuit.solarenergy.entity.TSamplingDataDetails
 */
@Mapper
public interface TSamplingDataDetailsMapper extends BaseMapper<TSamplingDataDetails> {


    /**
     * 获取无人机监测模块反演数据数据列表
     */
    Page<DetectionInvRet> getUAVInvDetectionData(Page page, @Param("uavDetectionReq") UAVDetectionReq uavDetectionReq);

    /**
     * 获取无人机监测模块原始影像数据列表
     */
    Page<DetectionOrgRet> getUAVOrgDetectionData(Page page, @Param("uavDetectionReq") UAVDetectionReq uavDetectionReq);

    /**
     * 获取多源遥感监测模块原始影像数据列表
     */
    Page<DetectionOrgRet> getMultiRSOrgDetectionData(Page page, @Param("multiRSDetectionReq") MultiRSDetectionReq multiRSDetectionReq);

    /**
     * 获取多源遥感监测模块反演影像数据列表
     */
    Page<DetectionInvRet> getMultiRSInvDetectionData(Page page, @Param("multiRSDetectionReq") MultiRSDetectionReq multiRSDetectionReq);

    /**
     * 方法功能：查询无人机（多源遥感）监测模块数据列表
     *
     * @param page
     * @param multiRSReq
     * @return
     * @date 2023/7/5 20:19
     * @author Tjb
     */


    Page<MapService> getDetectionData(Page page, @Param("multiRSReq") MultiRSReq multiRSReq);


    /**
     * @Description: 数据管理模块编辑接口
     * @Param: id, params
     * @return: Boolean
     * @Author: LuoSir
     * @Date: 2023/7/12
     */
    Boolean updateData(@Param("id") Integer id, @Param("params") Double params);

    /**
     * @Description: 数据管理删除接口
     * @Param: ids
     * @return: boolean
     * @Author: LuoSir
     * @Date: 2023/7/12
     */
    Integer updateDataByIds(@Param("ids") String ids);

    /**
     * @Description: 数据管理查询接口
     * @Param: ids
     * @return: boolean
     * @Author: LuoSir
     * @Date: 2023/7/12
     */
    Page<DataManageGetVoRet> getAllList(@Param("page") Page page, @Param("stTime") String stTime
            , @Param("endTime") String endTime, @Param("keywords") String keywords
            , @Param("collect") String collect);


    /**
     * 批量插入
     */
    Integer insertByBatch(List<TSamplingDataDetails> list);


    /**
     * @Description: 数据查询-采样点查询
     * @Param: DataQueryGetVoReq
     * @return: DataManageGetVoRetList
     * @Author: LuoSir
     * @Date: 2023/8/4
     */

    Page<StatisticQueryVoRet> getSamplePointData(@Param("page") Page page, @Param("stTime") String stTime, @Param("endTime") String endTime, @Param("keywords") String keywords, @Param("siteId") String siteId, @Param("streamName") String streamName);


    List<YearAndCountVoRet> getAllInfoForThreeYear(String streamName);
}




