package com.cuit.solarenergy.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.entity.TSiteBaseInfo;
import com.cuit.solarenergy.entity.vo.ret.MonitorSitesVo;
import com.cuit.solarenergy.entity.vo.ret.SiteAndValueVoRet;
import com.cuit.solarenergy.entity.vo.ret.SiteBaseInfoVoRet;
import com.cuit.solarenergy.vo.SiteBaseInfo;
import com.cuit.solarenergy.vo.SitesInfoForOne;
import com.cuit.solarenergy.vo.SitesInfosForOne;
import com.cuit.solarenergy.vo.param.SiteInfoParams;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
* @author PYJ
* @description 针对表【t_site_base_info(站点基本信息表)】的数据库操作Mapper
* @createDate 2023-06-08 11:37:12
* @Entity com.cuit.solarenergy.entity.TSiteBaseInfo
*/
@Mapper
public interface TSiteBaseInfoMapper extends BaseMapper<TSiteBaseInfo> {

    /**
     *方法功能：根据某一个水质参数查询当前年份所有站点的相关信息
     *
     * @param waterQualityParameter
     * @date 2023/6/24 10:20
     * @author Tjb
     * @return
     */

    List<SitesInfoForOne> getAllSiteForOne(String waterQualityParameter);

    /**
     *方法功能：获取某一流域指定时间段指定水质参数所有采样点详情
     *
     * @param siteInfoParams
     * @date 2023/6/24 19:26
     * @author Tjb
     * @return
     */

    Page<SitesInfosForOne> getStreamAllSitesInfosForOne(Page<SitesInfosForOne> page, @Param("siteInfoParams") SiteInfoParams siteInfoParams);

    /**
     *方法功能：根据站点id查询站点基本信息
     *
     * @param siteId
     * @date 2023/7/3 10:31
     * @author Tjb
     * @return
     */

    SiteBaseInfo getSiteBaseInfoById(String siteId);

    /**
     *方法功能：获取某一流域下所有的站点名称
     *
     * @param stream
     * @date 2023/7/3 16:05
     * @author Tjb
     * @return
     */

    List<String> getStationsForOneStream(String stream);

    /**
     *方法功能：获取监测数据首页指定流域指定水质参数当前年的站点信息List
     *
     * @param streamName
     * @param indexName
     * @date 2023/7/15 15:10
     * @author Tjb
     * @return
     */

    List<MonitorSitesVo> getMonitorSitesList(String streamName, String indexName);

    /**
     * @Description: 统计分析模块根据经纬度查询最近的三个站点
     * @return: List<TSiteBaseInfo>
     * @Author: LuoSir
     * @Date: 2023/7/17
     */

    List<TSiteBaseInfo> getSiteBaseInfoBySeparation();

//    /**
//     * @param paramsList，siteIdList
//     * @Description: 根据站点、水质参数获取曲线所需的列表
//     * @return:
//     * @Author: LuoSir
//     * @Date: 2023/8/7
//     */

//    List<StatisticQueryVoRet> queryDataCurveByParams(@Param("paramsList") String paramsList, @Param("siteIdList") String siteIdList);

    /**
     *
     * 返回所有站点经纬度
     */
    List<SiteBaseInfoVoRet> getAllSiteList();

    int editSite(String stationNo, Double lon, Double lat, String relativeSiteLocation, Double relativeSiteDistance, Integer distance);

    Page<TSiteBaseInfo> getList(Page<TSiteBaseInfo> page, String streamCode, String relativeLocation, String keywords);

    List<SiteAndValueVoRet> getValueOfSite(String streamName, String waterParameter);

}




