package com.cuit.solarenergy.minio;



import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.File;

@Service
public interface MinioServer {

    /**
     * 以File类形式上传文件
     *
     * @param file         文件对象
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    public String uploadFile(File file, String fileName, Boolean onlyFlag);
    /**
     * 以File类形式上传文件
     *
     * @param file         文件对象
     * @param filePath     文件存储路径，为空未指定时按日期自动生成
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */

    public String uploadFile(File file, String filePath, String fileName, Boolean onlyFlag) ;


    /**
     * 以MultipartFile类形式上传文件
     *
     * @param file         文件对象
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    public String uploadFile(MultipartFile file, String fileName, Boolean onlyFlag);

    /**
     * 以MultipartFile类形式上传文件
     *
     * @param file         文件对象
     * @param filePath     文件存储路径，为空未指定时按日期自动生成
     * @param fileName     文件存储名称
     * @param onlyFlag     文件名是否需要唯一 默认为 true - 需要唯一
     * @return minio对象唯一id
     */
    public String uploadFile(MultipartFile file, String filePath, String fileName, Boolean onlyFlag);


//    /**
//     * 获取原图可查看图片链接
//     *
//     * @param objectName minio对象唯一标识
//     * @param expireDayCount 设置过期时间（单位为秒）
//     * @return
//     */
//    String getShareURL(String objectName,int expireDayCount);
//
//
//    /**
//     * 获取压缩图可查看图片链接
//     *
//     * @param objectName
//     * @param expireDayCount 过期时间（单位为秒），默认为7天
//     * @return
//     */
//    String getCompressShareURL(String objectName,int expireDayCount);
//
//    /**
//     * 获取原图可查看图片链接（默认过期时间为7天）
//     *
//     * @param objectName minio对象唯一标识
//     * @return
//     */
//    String getShareURL(String objectName);

    /**
     * 获取压缩图可查看图片链接（永久有效）
     *
     * @param objectName minio对象唯一标识
     * @return 永久可访问链接
     */
    String getCompressPermanentShareURL(String objectName);



    /**
     * 获取原图可查看图片链接（永久有效）
     *
     * @param objectName minio对象唯一标识
     * @return  永久可访问链接
     */
    String getPermanentShareURL(String objectName);

    /**
     * 获取Minio地址（不包括服务器visitPoint)
     *
     * @param objectName minio对象唯一标识
     * @return  永久可访问链接
     */
    String getPermanentShareURLWithoutVisitPoint(String objectName);

//    /**
//     * 获取压缩图可查看图片链接（默认过期时间为7天）
//     *
//     * @param objectName
//     * @return
//     */
//    String getCompressShareURL(String objectName);


    /**
     * 删除文件
     *
     * @param objectName   minio对象唯一标识
     * @return
     */
    Boolean deleteFile(String objectName);

    /**
     * 下载文件，需指定下载路径
     * @param objectName minio对象唯一标识
     * @param savePath 保存路径
     * @return
     */
    Boolean downloadFile(String objectName, String savePath);


    /**
     * 下载文件，不需指定下载路径
     * @param objectName minio对象唯一标识
     * @param response
     */
    void downloadFile(String objectName, HttpServletResponse response) throws BcException;

}
