package com.cuit.solarenergy.minio;

import com.alibaba.fastjson.JSONObject;
import com.cuit.solarenergy.entity.TMinio;
import com.cuit.solarenergy.entity.vo.req.PicAviListGetVoReq;
import com.cuit.solarenergy.entity.vo.ret.PageVo;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;


@Service
public interface MinioService {

    /**
     * 以MultipartFile类形式上传文件
     *
     * @param file         文件对象
     *
     */
    MinioInfo uploadFile(MultipartFile file);

    /**
     * 以MultipartFile类形式上传文件对象
     *
     * @param file 文件对象
     * @param type 文件类型
     */
    MinioInfo uploadFile(MultipartFile file, String streamName, String siteName, String type);

    /**
     * 以MultipartFile类形式上传视频
     *
     * @param file         文件对象
     *
     */
    MinioInfo uploadVideo(MultipartFile file, String streamName, String siteName);

    /**
     * 以MultipartFile类形式上传tif文件
     *
     * @param file     文件对象(原tif文件)
     * @param file     预览图片文件对象
     * @param id       GeoServer发布的id
     * @param bbox     边框
     * @param max      最大值
     * @param min      最小值
     * @param avg      平均值
     * @param bands   波段
     */
    MinioInfo uploadForTif(MultipartFile file, MultipartFile pngfile, Long id, String bbox, Double max, Double min, Double avg, int bands) throws Exception;


    /**
     * 批量上传文件到文件服务器
     *
     * @param files
     * @return List<AppendixInfo>
     */
    List<MinioInfo> batchUploadFile(MultipartFile[] files);

    /**
     * 批量删除附件信息
     *
     * @param attachmentIds  附件ids
     * @return
     */
    Boolean deleteAppendix(String attachmentIds);

    /**
     * 文件上传（返回URL下载地址）
     *
     * @param fileName   文件名
     * @param file
     * @return: 文件url下载地址
     */
    JSONObject uploadByStream(String fileName, MultipartFile file) throws IOException;

    /**
     *  获取文件下载地址
     *
     * @param fileName   文件名
     * @param expires Expiry in seconds; defaults to 7 days.
     */
    JSONObject getFileUrl(String fileName, Integer expires);


    /**
     *  获取文件下载地址
     *
     * @param objectName   文件名
     */
    void downloadFile(String objectName, HttpServletResponse response) throws BcException;

    PageVo<TMinio> getPicAviList(PicAviListGetVoReq req);

    List<MinioInfo> uploadFiles(MultipartFile[] files);
}
