package com.cuit.solarenergy.service;

//import com.cuit.gis.analysis.spatialstatistics.RasterStatistic;
//import com.cuit.gis.exception.GeoTiffException;
//import com.cuit.gis.raster.GeoTiff;
//import com.cuit.gis.raster.GridCoverageUtil;
//import com.cuit.gis.raster.RasterBandExtraction;
//import org.geotools.coverage.grid.GridCoverage2D;
//
//import java.io.File;
//import java.io.IOException;
import java.util.Map;

/**
 * geotiff相关处理
 * @author cuitg -- ZengwuWang
 * @date 2023年08月02日 10:27
 */
public interface GeoTiffService {
    /**
     * 获取geotiff单波段文件中的最大、最小值等
     *
     * @param pathFilename 文件名（带路径和后缀名）
     * @return 统计Map Key:"count"、"min"、"max"、"sum"、"avg"
     */
    public Map<String, Double> getGeoTiffStatistic(String pathFilename);

    /**
     * 获取geotiff文件中的最大、最小值等
     *
     * @param pathFileName 文件名（带路径和后缀名）
     * @param band         波段
     * @return 统计Map Key:"count"、"min"、"max"、"sum"、"avg"
     */
    public Map<String, Double> getGeoTiffStatistic(String pathFileName, int band);

    /**
     * 提取geoServer中的某些波段生成新的文件
     *
     * @param pathFileName   源文件名（带路径和后缀名）
     * @param targetFileName 目标文件名（带路径和后缀名）
     * @param bands          波段（从0开始），如0或1，2，3。
     * @return
     */

    public boolean extractionGeoTiffBands(String pathFileName, String targetFileName, int... bands);

    /**
     * 取文件的波段数
     * @param pathFileName 源文件名（带路径和后缀名）
     * @return 波段数 <=0为异常
     */
    int getBands(String pathFileName);
}
