package com.cuit.solarenergy.service;

import com.cuit.solarenergy.base.BasePageQuery;
import com.cuit.solarenergy.entity.vo.ret.MonitorGeoURLAndBboxVoRet;
import com.cuit.solarenergy.entity.vo.ret.MonitorSitesVo;
import com.cuit.solarenergy.entity.vo.ret.PageVo;
import com.cuit.solarenergy.vo.SiteBaseInfo;
import com.cuit.solarenergy.vo.SitesInfoForOne;
import com.cuit.solarenergy.vo.SitesInfosForOne;
import com.cuit.solarenergy.vo.param.SiteInfoParams;

import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-06-17  09:51
 * @Description: 检测数据service
 * @Version: 1.0
 */
public interface MonitoringDataService {
    /**
     *
     * 根据某一个水质参数查询当前年份所有站点的相关信息
     */
    List<SitesInfoForOne> getAllSiteForOne(String waterQualityParameter);
    /**
     *
     * 获取某一流域指定时间段指定水质参数所有采样点详情
     */
    PageVo<SitesInfosForOne> getStreamAllSitesInfosForOne(SiteInfoParams siteInfoParams,BasePageQuery query);
    /**
     *
     * 根据站点id查询站点基本信息
     */
    SiteBaseInfo getSiteBaseInfoById(String siteId);
    /**
     *
     * 获取站点的所有图片
     */
    PageVo<String> getSitePictures(String siteId, BasePageQuery query);

    /**
     *
     * 获取站点的所有视频
     */
    PageVo<String> getSiteVideos(String siteId, BasePageQuery query);

    /**
     *
     * 获取监测数据首页指定流域指定水质参数当前年的站点信息List
     */
    List<MonitorSitesVo> getMonitorSitesList(String streamName, String indexName);

    /**
     *
     * 返回图层URL和bbox
     */
    List<MonitorGeoURLAndBboxVoRet> getGeoserverIdAndBbox(String streamCode, String typeCode, String dataCode) throws Exception;


    /**
     * 新增流域
     */
    Boolean addStream(String streamCode, String streamName) throws Exception;

    /**
     * 新增站点
     */
    Boolean addSite(String stationNo, String streamCode, Double lon, Double lat, String relativeSiteLocation, Double relativeSiteDistance, Integer distance ) throws Exception;

    /**
     * 修改流域
     */
    Boolean editStream(String streamCode, String streamName);

    /**
     * 修改站点
     */
    Boolean editSite(String stationNo, Double lon, Double lat, String relativeSiteLocation, Double relativeSiteDistance, Integer distance);
}
