package com.cuit.solarenergy.service.impl;


import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.entity.vo.req.DataManageGetVoReq;
import com.cuit.solarenergy.entity.vo.req.DataManageUpdateVoReq;
import com.cuit.solarenergy.entity.vo.req.DataQueryGetVoReq;
import com.cuit.solarenergy.entity.vo.ret.*;
import com.cuit.solarenergy.exception.ParamsException;
import com.cuit.solarenergy.mapper.TSamplingDataDetailsMapper;
import com.cuit.solarenergy.mapper.TSiteBaseInfoMapper;
import com.cuit.solarenergy.mapper.TStreamBaseInfoMapper;
import com.cuit.solarenergy.service.DataManagerService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-06  14:19
 * @Description: TODO
 * @Version: 1.0
 */
@Service
public class DataManagerServiceImpl implements DataManagerService {

    @Resource
    private TSamplingDataDetailsMapper samplingDataDetailsMapper;

    @Resource
    private TStreamBaseInfoMapper tStreamBaseInfoMapper;
    @Resource
    private TSiteBaseInfoMapper tSiteBaseInfoMapper;

    /**
     * @param req
     * @Description: 数据管理查询接口
     * @Param: dataManageGetVoReq, query
     * @return: PageVo
     * @Author: LuoSir
     * @Date: 2023/7/11
     */
    @Override
    public PageVo<DataManageGetVoRet> getList(DataManageGetVoReq req) {
        Page<DataManageGetVoRet> page = new Page<>(req.getCurrent(), req.getSize());
        page = samplingDataDetailsMapper.getAllList(page, req.getStTime(), req.getEndTime(), req.getKeywords(),
                req.getSiteIdList().stream().collect(Collectors.joining(",")));
        //将records数据取出来
        List<DataManageGetVoRet> recordslist = page.getRecords();
        if (recordslist.isEmpty()){
            throw new RuntimeException("没有符合条件的数据,请重新输入....");
        }
        //循环数据
        for (DataManageGetVoRet each : recordslist) {
            List<DataManageUpdateVoReq> list = new ArrayList<>();
            //取出records中的params数据，用分号分隔开存入eachS数组
            String[] eachS = each.getParams().split(";");
            //循环eachS，再按照,分隔，存入split数组中
            for (String s : eachS) {
                String[] split = s.split(",");
                //将分隔的字符串添加到list中
                list.add(new DataManageUpdateVoReq(Integer.valueOf(split[0]), split[1], Double.valueOf(split[2])));
            }
            each.setParamsList(list);
        }
        page.setRecords(recordslist);
        return new PageVo<>(page);
    }

    /**
     * @Description: 数据查询-采样点查询
     * @Param: DataQueryGetVoReq
     * @return: DataManageGetVoRetList
     * @Author: LuoSir
     * @Date: 2023/8/4
     */
    @Override
    public PageVo<StatisticQueryVoRet> getSamplePointData(DataQueryGetVoReq dataQueryGetVoReq) {
        Page<StatisticQueryVoRet> page = new Page<>(dataQueryGetVoReq.getCurrent(), dataQueryGetVoReq.getSize());
        page = samplingDataDetailsMapper.getSamplePointData(page, dataQueryGetVoReq.getStTime(), dataQueryGetVoReq.getEndTime()
                , dataQueryGetVoReq.getKeywords(),dataQueryGetVoReq.getSiteId(),dataQueryGetVoReq.getStreamName());
        //将records数据取出来
        List<StatisticQueryVoRet> recordslist = page.getRecords();
        if (recordslist.isEmpty()){
            throw new ParamsException("没有符合条件的数据,请重新筛选条件....");
        }
        //循环数据
        for (StatisticQueryVoRet statisticQueryVoRet : recordslist) {
            List<DataManageUpdateVoReq> list = new ArrayList<>();
            //取出records中的params数据，用分号分隔开存入eachS数组
            String[] eachS = statisticQueryVoRet.getParams().split(";");
            //循环eachS，再按照,分隔，存入split数组中
            for (String s : eachS) {
                String[] split = s.split(",");
                //将分隔的字符串添加到list中
                list.add(new DataManageUpdateVoReq(Integer.valueOf(split[0]), split[1], Double.valueOf(split[2])));
            }
            statisticQueryVoRet.setParamsList(list);
        }
        page.setRecords(recordslist);
        return new PageVo<>(page);
    }

    /**
     * @Description: 数据管理模块编辑接口
     * @Param: dataManageUpdateVoReqList
     * @return: Boolean
     * @Author: LuoSir
     * @Date: 2023/7/12
     */
    @Override
    public Boolean updateData(List<DataManageUpdateVoReq> dataManageUpdateVoReqList) {
        Boolean flag = false;
        //判断list是否为空,不为空则进行编辑操作
        if (!dataManageUpdateVoReqList.isEmpty() && dataManageUpdateVoReqList != null) {
            //循环传入的list，取到id和value（params）
            for (int i = 0; i < dataManageUpdateVoReqList.size(); i++) {
                flag = samplingDataDetailsMapper.updateData(dataManageUpdateVoReqList.get(i).getId(), dataManageUpdateVoReqList.get(i).getParams());
                if (flag == false) {
                    throw new ParamsException(dataManageUpdateVoReqList.get(i).getParams() + "编辑出现异常......");
                }
            }
            return flag;
        } else {
            throw new ParamsException("参数异常......");
        }
    }

    /**
     * @Description: 数据管理删除接口
     * @Param: ids
     * @return: boolean
     * @Author: LuoSir
     * @Date: 2023/7/12
     */
    @Override
    public Boolean deleteData(String ids) {

        Integer total = 0;
        total = samplingDataDetailsMapper.updateDataByIds(ids);
        if (total == 0) {
            throw new ParamsException("删除数据时出错......");
        }
        return true;
    }

    /**
     * @Description: 根据流域获取所有站点名
     * @Param:
     * @return: dataSiteByStreamRet
     * @Author: LuoSir
     * @Date: 2023/7/17
     */
    @Override
    public List<DataSiteTree> getSiteByStream() {
        List<DataSiteTree> dataSiteTreeList = new ArrayList<>();
        //1.取出所有流域名称
        List<Stream> allStreamName = tStreamBaseInfoMapper.getAllStreamName();
        for (Stream s : allStreamName) {
            //2.取出流域下所有站点名
            List<String> allSiteNameByStream = tSiteBaseInfoMapper.getStationsForOneStream(s.getStreamName());
            List<DataSiteTree> dataSiteTreeList1 = new ArrayList<>();
            for (String s1 : allSiteNameByStream) {
                dataSiteTreeList1.add(new DataSiteTree("站点", s1, null));
            }
            dataSiteTreeList.add(new DataSiteTree("流域", s.getStreamName(), dataSiteTreeList1));
        }
        return dataSiteTreeList;
    }


}
