package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.entity.vo.req.MultiRSDetectionReq;
import com.cuit.solarenergy.entity.vo.req.UAVDetectionReq;
import com.cuit.solarenergy.entity.vo.ret.DetectionInvRet;
import com.cuit.solarenergy.entity.vo.ret.DetectionOrgRet;
import com.cuit.solarenergy.entity.vo.ret.PageVo;
import com.cuit.solarenergy.mapper.TSamplingDataDetailsMapper;
import com.cuit.solarenergy.minio.MinioConfig;
import com.cuit.solarenergy.service.GeoServerPubService;
import com.cuit.solarenergy.service.MonitorService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.io.File;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-05  14:57
 * @Description: TODO
 * @Version: 1.0
 */
@Service
@Slf4j
public class MonitorServiceImpl implements MonitorService {

    @Autowired
    private TSamplingDataDetailsMapper tSamplingDataDetailsMapper;

    @Autowired
    private GeoServerPubService geoServerPubService;

    @Resource
    private MinioConfig properties;


    /**
     * 获取无人机监测模块反演数据数据列表
     */
    @Override
    public PageVo<DetectionInvRet> getUAVInvDetectionData(UAVDetectionReq uavDetectionReq) {

        Page<DetectionInvRet> detectionInvRetPage = new Page<>(uavDetectionReq.getCurrent(), uavDetectionReq.getSize());
        detectionInvRetPage = tSamplingDataDetailsMapper.getUAVInvDetectionData(detectionInvRetPage, uavDetectionReq);
        if (detectionInvRetPage != null) {
            for (int i = 0; i < detectionInvRetPage.getRecords().size(); i++) {
                detectionInvRetPage.getRecords().get(i).setGeoServerURL(geoServerPubService.getMapBox3857RasterWmsUrl(detectionInvRetPage.getRecords().get(i).getGeoId()));
                detectionInvRetPage.getRecords().get(i).setMinioURL(getMinioUrl(detectionInvRetPage.getRecords().get(i).getMinioURL()));
                detectionInvRetPage.getRecords().get(i).setPngMinioId(getMinioUrl(detectionInvRetPage.getRecords().get(i).getPngMinioId()));
                //                uavDetectionInvRetPage.getRecords().get(i).setGeoId(null);
            }
            return new PageVo<>(detectionInvRetPage);
        }
        return null;
    }


    /**
     * 获取无人机监测模块原始影像数据列表
     */
    @Override
    public PageVo<DetectionOrgRet> getUAVOrgDetectionData(UAVDetectionReq uavDetectionReq) {

        Page<DetectionOrgRet> detectionOrgRetPage = new Page<>(uavDetectionReq.getCurrent(), uavDetectionReq.getSize());
        detectionOrgRetPage = tSamplingDataDetailsMapper.getUAVOrgDetectionData(detectionOrgRetPage, uavDetectionReq);
        if (detectionOrgRetPage != null) {
            for (int i = 0; i < detectionOrgRetPage.getRecords().size(); i++) {
                detectionOrgRetPage.getRecords().get(i).setGeoServerURL(geoServerPubService.getMapBox3857RasterWmsUrl(detectionOrgRetPage.getRecords().get(i).getGeoId()));
                detectionOrgRetPage.getRecords().get(i).setMinioURL(getMinioUrl(detectionOrgRetPage.getRecords().get(i).getMinioURL()));
                detectionOrgRetPage.getRecords().get(i).setPngMinioId(getMinioUrl(detectionOrgRetPage.getRecords().get(i).getPngMinioId()));
                //uavDetectionOrgRetPage.getRecords().get(i).setGeoId(null);
            }
            return new PageVo<>(detectionOrgRetPage);
        }
        return null;
    }


    /**
     * 获取多源遥感监测模块反演数据数据列表
     */
    @Override
    public PageVo<DetectionInvRet> getMultiRSInvDetectionData(MultiRSDetectionReq multiRSDetectionReq) {

        Page<DetectionInvRet> detectionInvRetPage = new Page<>(multiRSDetectionReq.getCurrent(), multiRSDetectionReq.getSize());
        detectionInvRetPage = tSamplingDataDetailsMapper.getMultiRSInvDetectionData(detectionInvRetPage, multiRSDetectionReq);
        if (detectionInvRetPage != null) {
            for (int i = 0; i < detectionInvRetPage.getRecords().size(); i++) {
                detectionInvRetPage.getRecords().get(i).setGeoServerURL(geoServerPubService.getMapBox3857RasterWmsUrl(detectionInvRetPage.getRecords().get(i).getGeoId()));
                detectionInvRetPage.getRecords().get(i).setMinioURL(getMinioUrl(detectionInvRetPage.getRecords().get(i).getMinioURL()));
                detectionInvRetPage.getRecords().get(i).setPngMinioId(getMinioUrl(detectionInvRetPage.getRecords().get(i).getPngMinioId()));
                //  uavDetectionInvRetPage.getRecords().get(i).setGeoId(null);
            }
            return new PageVo<>(detectionInvRetPage);
        }
        return null;
    }


    /**
     * 获取多源遥感监测模块原始影像数据列表
     */
    @Override
    public PageVo<DetectionOrgRet> getMultiRSOrgDetectionData(MultiRSDetectionReq multiRSDetectionReq) {

        Page<DetectionOrgRet> detectionOrgRetPage = new Page<>(multiRSDetectionReq.getCurrent(), multiRSDetectionReq.getSize());
        detectionOrgRetPage = tSamplingDataDetailsMapper.getMultiRSOrgDetectionData(detectionOrgRetPage, multiRSDetectionReq);
        if (detectionOrgRetPage != null) {
            for (int i = 0; i < detectionOrgRetPage.getRecords().size(); i++) {
                detectionOrgRetPage.getRecords().get(i).setGeoServerURL(geoServerPubService.getMapBox3857RasterWmsUrl(detectionOrgRetPage.getRecords().get(i).getGeoId()));
                detectionOrgRetPage.getRecords().get(i).setMinioURL(getMinioUrl(detectionOrgRetPage.getRecords().get(i).getMinioURL()));
                detectionOrgRetPage.getRecords().get(i).setPngMinioId(getMinioUrl(detectionOrgRetPage.getRecords().get(i).getPngMinioId()));
                //                uavDetectionOrgRetPage.getRecords().get(i).setGeoId(null);
            }
            return new PageVo<>(detectionOrgRetPage);
        }
        return null;
    }

    private String getMinioUrl(String minioID) {
        String url = properties.getVisitPoint();
        if (org.apache.commons.lang3.StringUtils.isEmpty(minioID)) {
            return minioID;
        }
        if (!(url.endsWith(File.separator) && url.endsWith(File.pathSeparator))) {
            url = url + File.separator;
        }
        String miniourl = url + minioID;
        return miniourl;
    }
}
