package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.common.dto.SysLogDto;
import com.cuit.solarenergy.entity.sys.SysLog;
import com.cuit.solarenergy.mapper.SysLogMapper;
import com.cuit.solarenergy.model.user.SysLogQuery;
import com.cuit.solarenergy.service.SysLogService;
import com.cuit.solarenergy.utils.ErrorMessage;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class SysLogServiceImpl implements SysLogService {

    @Resource
    private SysLogMapper logMapper;

    @Override
    public Map querySysLogByParams(SysLogQuery sysLogQuery) {
        Map refMap = new HashMap();
        try {
            // 开启分页
            log.debug("开始分页查询：分页当前页数{"+sysLogQuery.getCurrent()+"}，每页条数为{"+sysLogQuery.getSize()+"}");
            Page<SysLogDto> pageInfo = new Page<>(sysLogQuery.getCurrent(), sysLogQuery.getSize());
            // 得到对应分页对象
             logMapper.querySysLogByQuery(pageInfo, sysLogQuery.getStTime(), sysLogQuery.getEndTime(), sysLogQuery.getUserName(), sysLogQuery.getIp(),sysLogQuery.getModule(),sysLogQuery.getOpType());
            // 设置分页好的列表
            refMap.put("data",pageInfo.getRecords());
            refMap.put("count",pageInfo.getPages());
            return refMap;
        } catch (Exception e) {
            log.error("分页查询失败失败...");
            e.printStackTrace();
            throw new ErrorMessage("查询数据失败！");
        }
    }

    @Override
    public Boolean writeModelLog(String message, String ip, LocalDateTime modelStart, LocalDateTime modelEnd) {
        try {
            logMapper.insert(SysLog.builder().isValid(1)
                    .module("模型服务")
                    .opration(message)
                    .visitTime(modelStart)
                    .opType(fetchOptype(message))
                    .method("writeModelLog")
                    .ip(ip)
                    .executionTime(Duration.between(modelStart, modelEnd).toMillis() + "毫秒")
                    .userName("模型服务器")
                    .build());
        }catch (Exception e) {
            throw new ErrorMessage("模型日志记录失败!");
        }
        return true;
    }

    private Integer fetchOptype(String message) {
        switch (message) {
            case "model_run" :
                return 9;
            case "param_modify" :
                return 10;
            case "article_create" :
                return 11;
            default:
                return null;
        }
    }
}
