package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.cuit.solarenergy.base.BasePageQuery;
import com.cuit.solarenergy.entity.Model;
import com.cuit.solarenergy.entity.User;
import com.cuit.solarenergy.entity.UserRole;
import com.cuit.solarenergy.entity.vo.req.SystemManageModelVoReq;
import com.cuit.solarenergy.entity.vo.req.UserUpdateVoReq;
import com.cuit.solarenergy.entity.vo.ret.PageVo;
import com.cuit.solarenergy.entity.vo.ret.RoleManageVoRet;
import com.cuit.solarenergy.entity.vo.ret.UserManageVoRet;
import com.cuit.solarenergy.mapper.ModelMapper;
import com.cuit.solarenergy.mapper.RoleMapper;
import com.cuit.solarenergy.mapper.UserMapper;
import com.cuit.solarenergy.mapper.UserRoleMapper;
import com.cuit.solarenergy.service.SystemManageService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * @Author: Tjb
 * @CreateTime: 2023-07-17  10:23
 * @Description: TODO
 * @Version: 1.0
 */
@Service
@Slf4j
public class SystemManageServiceImpl implements SystemManageService {

    @Autowired
    private RoleMapper roleMapper;

    @Autowired
    private UserMapper userMapper;

    @Autowired
    private UserRoleMapper userRoleMapper;

    @Autowired
    private ModelMapper modelMapper;

    @Autowired
    private AuthenticationManager authenticationManager;

//    @Autowired
//    private TModelMapper tModelMapper;

    //    @Autowired
//    private BCryptPasswordEncoder passwordEncoder;
    BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();

    /**
     * 方法功能：获取系统管理-角色管理list
     *
     * @param
     * @return
     * @date 2023/7/17 10:24
     * @author Tjb
     */

    @Override
    public List<RoleManageVoRet> getRoleManageList() {
        return roleMapper.getRoleManageList();
    }

    /**
     * 方法功能：获取系统管理-用户管理list
     *
     * @param query
     * @return
     * @date 2023/7/17 14:58
     * @author Tjb
     */

    @Override
    public PageVo<UserManageVoRet> getUserManageList(BasePageQuery query) {
        Page<UserManageVoRet> page = new Page<>(query.getCurrent(), query.getSize());
        page = roleMapper.getUserManageList(page);
        return new PageVo<>(page);
    }

    /**
     * 方法功能：系统管理-用户管理-更新用户信息
     *
     * @param userUpdateVoReq@return
     * @date 2023/7/17 15:45
     * @author Tjb
     */
    @Transactional
    @Override
    public Boolean updateUser(UserUpdateVoReq userUpdateVoReq) {
        log.info("更新 {} 的用户信息！", userUpdateVoReq);
        User oldUser = userMapper.selectById(userUpdateVoReq.getId());
        //获取用户已有权限
        String oldUserType = oldUser.getUserType();//0：管理员，1：普通用户
        //权限不变
        if (Integer.parseInt(oldUserType) == Integer.parseInt(userUpdateVoReq.getUserType())) {
            int index = userMapper.updateUser(userUpdateVoReq);
            if (index != 0) {
                return true;
            } else {
                throw new RuntimeException("更新用户失败！");
            }
        } else {//权限改变
            if (Integer.parseInt(userUpdateVoReq.getUserType()) == 1) {//由管理员变为普通用户
                userRoleMapper.updateById(new UserRole(userUpdateVoReq.getId(), new Long(2)));
                log.info(" {} 由管理员变为普通用户成功！", userUpdateVoReq.getUserName());
                int index = userMapper.updateUser(userUpdateVoReq);
                if (index != 0) {
                    return true;
                } else {
                    throw new RuntimeException("更新用户失败！");
                }
            }
            if (Integer.parseInt(userUpdateVoReq.getUserType()) == 0) {//由普通用户变为管理员
                userRoleMapper.updateById(new UserRole(userUpdateVoReq.getId(), new Long(1)));
                log.info(" {} 由普通用户变为管理员成功！", userUpdateVoReq.getUserName());
                int index = userMapper.updateUser(userUpdateVoReq);
                if (index != 0) {
                    return true;
                } else {
                    throw new RuntimeException("更新用户失败！");
                }
            } else {
                throw new RuntimeException("更新用户角色失败！");
            }
        }

    }

    /**
     * 方法功能：系统管理-用户管理-删除用户信息
     *
     * @param id
     * @return
     * @date 2023/7/17 15:59
     * @author Tjb
     */

    @Override
    public Boolean deleteUser(String id) {
        log.info("删除id为 {} 的用户信息！", id);
        int index = userMapper.deleteById(id);
        if (index != 0) {
            return true;
        } else {
            throw new RuntimeException("删除用户失败！");
        }
    }

    @Override
    public Boolean changeUserPassword(Long id, String oldPassword, String newPassword) {
        log.info("更新 {} 的用户密码！", id);
        User user = userMapper.selectById(id);
        return changePassword(oldPassword, newPassword, user);
    }

    @Override
    public Boolean changeUserPassword(String name, String oldPassword, String newPassword) {
        //校验用户名
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserName, name);
        User user = userMapper.selectOne(lambdaQueryWrapper);

        return changePassword(oldPassword, newPassword, user);
    }

    private boolean changePassword(String oldPassword, String newPassword, User user) {

        //密码验证
        if (!passwordEncoder.matches(oldPassword, user.getPassword())) {
            throw new RuntimeException("原始密码不正确！");
        }

        //密码加密
        String newEncoderPassword = passwordEncoder.encode(newPassword);
        user.setPassword(newEncoderPassword);
        int upnum = userMapper.updateById(user);

        if (upnum == 1) {
            return true;
        } else {
            throw new RuntimeException("更新密码失败！");
        }
    }



    /**
     * 方法功能：系统管理-用户管理-新增用户
     *
     * @param user
     * @return
     * @date 2023/7/17 16:04
     * @author Tjb
     */

    @Transactional
    @Override
    public Boolean addUser(User user) {
        //校验用户名是否可用
        LambdaQueryWrapper<User> lambdaQueryWrapper = new LambdaQueryWrapper<>();
        lambdaQueryWrapper.eq(User::getUserName, user.getUserName());
        User userByName = userMapper.selectOne(lambdaQueryWrapper);
        if (userByName != null) {
            throw new RuntimeException("用户名不可用！");
        }
        log.info("新增用户：{}", user);
        //密码加密
        String lockPassword = passwordEncoder.encode(user.getPassword());
        user.setPassword(lockPassword);
        int index = userMapper.insert(user);
        if (index != 0) {
            //取出表中user的id，并赋予对应的权限
            LambdaQueryWrapper<User> userQueryWrapper = new LambdaQueryWrapper<>();
            userQueryWrapper.eq(User::getUserName, user.getUserName());
            userQueryWrapper.eq(User::getPassword, lockPassword);
            User selectOne = userMapper.selectOne(userQueryWrapper);
            //获取新增用户id
            Long userId = selectOne.getId();

            if (Integer.parseInt(user.getUserType()) == 0) {//赋予管理员权限
                userRoleMapper.insert(new UserRole(userId, new Long(1)));
                log.info("赋予 {} 管理员权限成功！", user.getUserName());
            }
            if (Integer.parseInt(user.getUserType()) == 1) {//赋予普通用户权限
                userRoleMapper.insert(new UserRole(userId, new Long(2)));
                log.info("赋予 {} 普通用户权限成功！", user.getUserName());
            }
            return true;
        } else {
            throw new RuntimeException("新增用户失败！");
        }
    }

    /**
     * 方法功能：获取系统管理-用户管理关键字查询list
     *
     * @param query
     * @return
     * @date 2023/7/17 19:50
     * @author Tjb
     */

    @Override
    public PageVo<UserManageVoRet> getUserManageListBySearch(BasePageQuery query, String keyWord) {
        log.info("查找用户名关键字为 {} 的用户列表", keyWord);
        Page<UserManageVoRet> page = new Page<>(query.getCurrent(), query.getSize());
        page = roleMapper.getUserManageListBySearch(page, keyWord);
        return new PageVo<>(page);
    }

    /**
     * 方法功能：获取用户角色list
     *
     * @param
     * @return
     * @date 2023/7/17 20:10
     * @author Tjb
     */

    @Override
    public ArrayList<String> getUserRole() {
        ArrayList<String> getUserRoleList;
        getUserRoleList = userMapper.getUserRole();
        return getUserRoleList;
    }

    /**
     * 方法功能：保存模型参数
     *
     * @param systemManageModelVoReq
     * @return
     * @date 2023/7/24 15:01
     * @author Tjb
     */
    @Transactional
    @Override
    public Boolean saveModel(SystemManageModelVoReq systemManageModelVoReq) {
        int satelliteInsert = 0;
        int uavInsert = 0;
        log.info("将模型参数类 {} 保存到数据库中！", systemManageModelVoReq);
        if (systemManageModelVoReq.getSatelliteA() != null && systemManageModelVoReq.getSatelliteB() != null && systemManageModelVoReq.getSatelliteC() != null) {
            Model satelliteModel = Model.builder()
                    .indexName(systemManageModelVoReq.getIndex())
                    .typeNumber(new Integer(0))
                    .satellite(systemManageModelVoReq.getSatellite())
                    .firstVal(systemManageModelVoReq.getSatelliteA())
                    .secondVal(systemManageModelVoReq.getSatelliteB())
                    .thirdVal(systemManageModelVoReq.getSatelliteC())
                    .build();
            //将卫星反演模型数据入库
            satelliteInsert = modelMapper.insert(satelliteModel);
        } else if (systemManageModelVoReq.getUavA() != null && systemManageModelVoReq.getUavB() != null && systemManageModelVoReq.getUavC() != null) {
            Model uavModel = Model.builder()
                    .indexName(systemManageModelVoReq.getIndex())
                    .typeNumber(new Integer(1))
                    .firstVal(systemManageModelVoReq.getUavA())
                    .secondVal(systemManageModelVoReq.getUavB())
                    .thirdVal(systemManageModelVoReq.getUavC())
                    .build();

            //将无人机反演模型数据入库
            uavInsert = modelMapper.insert(uavModel);
        }
//        int satelliteInsert = tModelMapper.saveModel(satelliteModel);

//        int uavInsert = tModelMapper.saveModel(uavModel);
        if (satelliteInsert != 0 || uavInsert != 0) {
            log.info("模型参数保存成功！");
            return true;
        } else {
            log.info("模型参数保存失败！");
            return false;
        }
    }

    @Override
    public Boolean checkPasswordComplexity(String password) {
        // 密码长度至少为8个字符
        if (password == null || password.length() < 8) {
            return false;
        }

        // 定义正则表达式，要求至少包含一个小写字母，一个大写字母，一个数字和一个特殊字符
        String regex = "(?=.*[a-z])(?=.*[A-Z])(?=.*\\d)(?=.*[!@#$%^&*()_+=<>?{|}~-]).{8,}";

        return password.matches(regex);
    }
}

