package com.cuit.solarenergy.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.cuit.gis.manager.MeteGis;
import com.cuit.solarenergy.config.MyGeoServerConfig;
import com.cuit.solarenergy.entity.*;
import com.cuit.solarenergy.mapper.GeoserverinfoMapper;
import com.cuit.solarenergy.mapper.SatelliteSendDataMapper;
import com.cuit.solarenergy.minio.MinioConfig;
import com.cuit.solarenergy.minio.MinioServer;
import com.cuit.solarenergy.minio.MinioService;
import com.cuit.solarenergy.service.*;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.entity.ContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.*;

/**
 * Tif数据处理
 * @author cuitg -- ZengwuWang
 * @date 2024年04月28日 21:10
 */
@Service
@Slf4j
public class TifDataDealServiceImpl implements TifDataDealService {
    @Autowired
    MyGeoServerConfig myGeoServerConfig;

    @Resource
    GeoServerPubService geoServerPubService;
    @Resource
    GeoTiffService geoTiffService;
    @Resource
    GeoserverinfoMapper geoserverinfoMapper;
    @Resource
    MinioService minioService;

    @Resource
    SatelliteDataService satelliteDataService;
    @Resource
    SatelliteInvDataService satelliteInvDataService;
    @Resource
    SatelliteSendDataService satelliteSendDataService;

    @Resource
    UavDataService uavDataService;
    @Resource
    UavInvDataService uavInvDataService;
    @Resource
    UavSendDataService uavSendDataService;

    @Resource
    GeoserverinfoService geoserverinfoService;

    @Resource
    private SatelliteSendDataMapper satelliteSendDataMapper;

    @Resource
    MinioServer minioServer;

    @Resource
    private MinioConfig properties;

    @Override
    public int dealTif2GeoserverAndMinioAndDB(String pathFileName) {
        MeteGis.init();


        String styleName = "style_RGB";

        String temppath = "temp" + File.separator + "png_"+UUID.randomUUID()+ File.separator;
        //        //文件夹
        File folder = new File(temppath);
        //判断文件是否存在
        if (!folder.exists()){//不存在，则创建
            //这里需要使用mkdirs，因为为多层目录
            folder.mkdirs();
        }


        int ret = dealTif2GeoserverAndMinioAndDB(pathFileName, styleName, temppath);

        //每结束一个删除临时文件夹下的所有文件
        deleteDirectory(temppath);
        return ret;
    }

    private int dealTif2GeoserverAndMinioAndDB(String pathFileName, String styleName, String temppath) {
        int ret = 0;
        Double max = null;
        Double min = null;
        Double avg = null;

        Map<String, Double> geoTiffStatistic = geoTiffService.getGeoTiffStatistic(pathFileName, 0);

        if (geoTiffStatistic != null) {
            max = geoTiffStatistic.get("max");
            min = geoTiffStatistic.get("min");
            avg = geoTiffStatistic.get("avg");

            //先发布GeoServer并保存t_geoserver
            Long aLong = geoServerPubService.pubGeoServerAndSavetoDB(styleName, "EPSG:3857", pathFileName);
            if (aLong != null && aLong != -1) {

                //生成预览图片
                Geoserverinfo geoserverinfo = geoserverinfoMapper.selectById(aLong);
                //bbox
                String bbox = geoserverinfo.getBbox();

                //用于存放临时Png的唯一目录
                String tempPngPath = geoserverinfo.getGeoserverlayers();

                //png
                String tempFile = temppath + File.separator + "ys" + tempPngPath + ".png";
                boolean pngFlag = geoServerPubService.getPng(aLong, 100, 60, tempFile);
                if (pngFlag) {
                    //生成生成预览图片成功
                }

                //获取文件的波段数
                int fileBands = geoTiffService.getBands(pathFileName);

                FileInputStream pngfileInputStream = null;
                FileInputStream fileInputStream = null;
                try {

                    //File 转 MultipartFile
                    File pdfFile = new File(pathFileName);
                    fileInputStream = new FileInputStream(pdfFile);
                    MultipartFile multipartFile = new MockMultipartFile(pdfFile.getName(), pdfFile.getName(),
                            ContentType.APPLICATION_OCTET_STREAM.toString(), fileInputStream);

                    File pngFile = new File(tempFile);
                    pngfileInputStream = new FileInputStream(pngFile);
                    MultipartFile pngmultipartFile = new MockMultipartFile(pngFile.getName(), pngFile.getName(),
                            ContentType.APPLICATION_OCTET_STREAM.toString(), pngfileInputStream);

                    //上传原始数据及预览图片到minio
                    minioService.uploadForTif(multipartFile, pngmultipartFile, aLong, bbox, max, min, avg, fileBands);

                    log.info("上传文件{}到Minio完成",  pathFileName);
                    ret = 1;

                } catch (Exception e) {
                    log.error("上传文件{}到Minio时，出错：{}",  pathFileName, e.getMessage());
                } finally {
                    if (pngfileInputStream != null) {
                        try {
                            pngfileInputStream.close();
                        } catch (IOException e) {
                            log.warn("上传文件{}到Minio时，关闭文件流{}时出错！", pathFileName, tempFile, e.getMessage());
                        }
                    }
                    if (fileInputStream != null) {
                        try {
                            fileInputStream.close();
                        } catch (IOException e) {
                            log.warn("上传文件{}到Minio时，关闭文件流时出错！", pathFileName, e.getMessage());
                        }
                    }
                }
            }
        }
        return ret;
    }

    @Override
    public List<String> batchDealTif2GeoserverAndMinioAndDB(String []paths) {
        MeteGis.init();
        int ret = 0;
        List<String> retMsg = new ArrayList<>();

        Map<String, String> map = new LinkedHashMap<>();

        map.put("RGB", "style_RGB");
//        map.put("DOM", "style_DOM");
        map.put("ORD", "style_DOM");
        map.put("IR", "style_IR");
        map.put("NIR", "style_NIR");
        map.put("CPC", "style_CPC");
        map.put("NTS", "style_NTS");
        map.put("SPM", "style_SPM");
        map.put("TBD", "style_TBD");
        map.put("TNG", "style_TNG");
        map.put("TSP", "style_TSP");

        String style = null;


        String temppath = "temp" + File.separator + "png_" + UUID.randomUUID() + File.separator;
        //        //文件夹
        File folder = new File(temppath);
        //判断文件是否存在
        if (!folder.exists()) {//不存在，则创建
            //这里需要使用mkdirs，因为为多层目录
            folder.mkdirs();
        }

        Runtime r = Runtime.getRuntime();
        r.gc();   //垃圾回收


        log.info("正在处理，文件个数：" + paths.length);

        for (int i = 0; i < paths.length; i++) {
            long start = System.currentTimeMillis(); //开始时间
            long end = 0;
            long begusedMemory = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
            long endusedMemory = 0;

            String pathFileName = paths[i];



            String filename = StringUtils.getFilename(StringUtils.cleanPath(pathFileName));

            if (!(filename.startsWith("SAT_ORD_") || filename.startsWith("SAT_RTD_") ||
                    filename.startsWith("UAV_ORD_") || filename.startsWith("UAV_RTD_"))) {
                log.info("文件：" + (i + 1) + "/" + paths.length + ":" + filename + "文件名格式不对！");
                continue;
            }
            log.info("正在处理文件：" + pathFileName);

            String[] split = filename.split("_");

            if (split.length > 0) {
                if ("SAT".equals(split[0]) && "ORD".equals(split[1])) {
                    style = "style_RGB";
                } else if ("UAV".equals(split[0]) && "ORD".equals(split[1])) {
                    style = "style_DOM";
                }
                if ("RTD".equals(split[1])) {
                    style = map.get(split[4]);
                }
            }
            if (style == null) {
                log.info("文件：" + (i + 1) + "/" + paths.length + ":" + pathFileName + "文件名格式不对！未包括 ORD / RTD或类型");
                continue;
            }

            int num = dealTif2GeoserverAndMinioAndDB(pathFileName, style, temppath);
            ret += num;

            endusedMemory = Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory();
            end = System.currentTimeMillis();
            log.debug("上传文件Minio生成完成， 用时：" + (end - start) * 0.001 / 60 + "分钟");
            log.debug("内存消耗(新增):" + (endusedMemory - begusedMemory) / 1024 + "KB");
            retMsg.add("处理文件" + filename + (num == 1 ? "成功" : "失败"));

        }

        //每结束一个删除临时文件夹下的所有文件
        deleteDirectory(temppath);

        return retMsg;
    }

    @Override
    public int deleteTif2GeoserverAndMinioAndDB() {
        int num = 0;
        //---------------- 卫星--原始-------------------
        num = num + deleteStaData();

        //---------------- 卫星--反演-------------------
        num = num +deleteStaInvData();

        //---------------- 无人机--原始-------------------
        num = num + deleteUavData();

        //---------------- 无人机--反演-------------------
        num = num +deleteUavInvData();

        return num;
    }

    private int deleteStaInvData() {
        int delenum = 0;
        //从数据库中查询所有数据
        List<SatelliteInvData> satelliteInvDataList = satelliteInvDataService.list();
        for (int i = 0; i < satelliteInvDataList.size(); i++) {
            SatelliteInvData satelliteInvData = satelliteInvDataList.get(i);
            //删除GeoServer数据
            Geoserverinfo geoserverinfo = geoserverinfoService.getById(satelliteInvData.getGeoserverInfoId());

            String geoserverworkspaces = geoserverinfo.getGeoserverworkspaces();
            String geoserverlayers = geoserverinfo.getGeoserverlayers();
            //删除图层
            Boolean deleFlag = geoServerPubService.removeLayers(geoserverworkspaces, geoserverlayers);
            //删除图层目录数据
            String geoServerPath = myGeoServerConfig.getGeoServerPath();
            String dataPath = geoServerPath + File.separator + geoserverinfo.getGeoserverlayers();
            //删除文件
            deleteDirectory(dataPath);


            //删除minio
            String minioId = satelliteInvData.getMinioId();
            String objectName = "";
            String prefix = properties.getVisitPoint() + "/" + properties.getBucketName() + "/";
            if (minioId.startsWith(prefix)) {
                //保存的地址
                objectName = minioId.substring(prefix.length());
            } else {
                objectName = minioId;
            }
            try {
                minioServer.deleteFile(objectName);
            }catch (Exception e) {
                log.warn("删除Minio文件"+objectName+"失败");
            }

            //删除minio -- PNG
            String pngMinioId = satelliteInvData.getPngMinioId();
            if (pngMinioId.startsWith(prefix)) {
                //保存的地址
                objectName = pngMinioId.substring(prefix.length());
            } else {
                objectName = pngMinioId;
            }
            try {
                minioServer.deleteFile(objectName);
            }catch (Exception e) {
                log.warn("删除Minio文件"+objectName+"失败");
            }

            //删除geoserver表中数据
            geoserverinfoService.removeById(geoserverinfo.getId());

            //删除数据库中数据
            satelliteInvDataService.removeById(satelliteInvData.getId());
            delenum ++;
        }
        return delenum;
    }

    private int deleteStaData() {
        int delenum = 0;
        //从数据库中查询所有数据
        List<SatelliteData> listSatelliteData = satelliteDataService.list();
        for (int i = 0; i < listSatelliteData.size(); i++) {
            SatelliteData satelliteData = listSatelliteData.get(i);
            //删除GeoServer数据
            LambdaQueryWrapper<SatelliteSendData> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(SatelliteSendData::getPid, satelliteData.getId());
            List<SatelliteSendData> listSatelliteSendData = satelliteSendDataService.list(queryWrapper);
            //取发布数据
            for (int j = 0; j < listSatelliteSendData.size(); j++) {
                SatelliteSendData satelliteSendData = listSatelliteSendData.get(j);
                //取geoserverid
                Geoserverinfo geoserverinfo = geoserverinfoService.getById(satelliteSendData.getGeoserverInfoId());
                String geoserverworkspaces = geoserverinfo.getGeoserverworkspaces();
                String geoserverlayers = geoserverinfo.getGeoserverlayers();
                //删除图层
                Boolean deleFlag = geoServerPubService.removeLayers(geoserverworkspaces, geoserverlayers);
                //删除图层目录数据
                String geoServerPath = myGeoServerConfig.getGeoServerPath();
                String dataPath = geoServerPath + File.separator + geoserverinfo.getGeoserverlayers();

                //删除文件
                deleteDirectory(dataPath);

                //删除minio
                String pngMinioId = satelliteSendData.getPngMinioId();
                String objectName = "";
                String prefix = properties.getVisitPoint() + "/" + properties.getBucketName() + "/";
                if (pngMinioId.startsWith(prefix)) {
                    //保存的地址
                    objectName = pngMinioId.substring(prefix.length());
                } else {
                    objectName = pngMinioId;
                }
                try {
                    minioServer.deleteFile(objectName);
                }catch (Exception e) {
                    log.warn("删除Minio文件"+objectName+"失败");
                }

                //删除Geoserver中的数据
                geoserverinfoService.removeById(satelliteSendData.getGeoserverInfoId());

                //删除数据库中数据
                satelliteSendDataService.removeById(satelliteSendData.getId());

                delenum ++;
            }
            //删除Minioo数据
            String minioId = satelliteData.getMinioId();
            String objectName = "";
            String prefix = properties.getVisitPoint() + "/" + properties.getBucketName() + "/";
            if (minioId.startsWith(prefix)) {
                //保存的地址
                objectName = minioId.substring(prefix.length());
            } else {
                objectName = minioId;
            }
            minioServer.deleteFile(objectName);
            //删除数据库中satelliteData数据
            satelliteDataService.removeById(satelliteData.getId());
        }
        return delenum;
    }

    private int deleteUavInvData() {
        int delenum = 0;
        //从数据库中查询所有数据
        List<UavInvData> uavInvDataList = uavInvDataService.list();
        for (int i = 0; i < uavInvDataList.size(); i++) {
            UavInvData uavInvData = uavInvDataList.get(i);
            //删除GeoServer数据
            Geoserverinfo geoserverinfo = geoserverinfoService.getById(uavInvData.getGeoserverInfoId());

            String geoserverworkspaces = geoserverinfo.getGeoserverworkspaces();
            String geoserverlayers = geoserverinfo.getGeoserverlayers();
            //删除图层
            Boolean deleFlag = geoServerPubService.removeLayers(geoserverworkspaces, geoserverlayers);
            //删除图层目录数据
            String geoServerPath = myGeoServerConfig.getGeoServerPath();
            String dataPath = geoServerPath + File.separator + geoserverinfo.getGeoserverlayers();
            //删除文件
            deleteDirectory(dataPath);


            //删除minio
            String minioId = uavInvData.getMinioId();
            String objectName = "";
            String prefix = properties.getVisitPoint() + "/" + properties.getBucketName() + "/";
            if (minioId.startsWith(prefix)) {
                //保存的地址
                objectName = minioId.substring(prefix.length());
            } else {
                objectName = minioId;
            }
            try {
                minioServer.deleteFile(objectName);
            }catch (Exception e) {
                log.warn("删除Minio文件"+objectName+"失败");
            }

            //删除minio -- PNG
            String pngMinioId = uavInvData.getPngMinioId();
            if (pngMinioId.startsWith(prefix)) {
                //保存的地址
                objectName = pngMinioId.substring(prefix.length());
            } else {
                objectName = pngMinioId;
            }
            try {
                minioServer.deleteFile(objectName);
            }catch (Exception e) {
                log.warn("删除Minio文件"+objectName+"失败");
            }

            //删除geoserver表中数据
            geoserverinfoService.removeById(geoserverinfo.getId());

            //删除数据库中数据
            uavInvDataService.removeById(uavInvData.getId());
            delenum ++;
        }
        return delenum;
    }

    private int deleteUavData() {
        int delenum = 0;
        //从数据库中查询所有数据
        List<UavData> listSatelliteData = uavDataService.list();
        for (int i = 0; i < listSatelliteData.size(); i++) {
            UavData uavData = listSatelliteData.get(i);
            //删除GeoServer数据
            LambdaQueryWrapper<UavSendData> queryWrapper = new LambdaQueryWrapper<>();
            queryWrapper.eq(UavSendData::getPid, uavData.getId());
            List<UavSendData> listSatelliteSendData = uavSendDataService.list(queryWrapper);
            //取发布数据
            for (int j = 0; j < listSatelliteSendData.size(); j++) {
                UavSendData uavSendData = listSatelliteSendData.get(j);
                //取geoserverid
                Geoserverinfo geoserverinfo = geoserverinfoService.getById(uavSendData.getGeoserverInfoId());
                String geoserverworkspaces = geoserverinfo.getGeoserverworkspaces();
                String geoserverlayers = geoserverinfo.getGeoserverlayers();
                //删除图层
                Boolean deleFlag = geoServerPubService.removeLayers(geoserverworkspaces, geoserverlayers);
                //删除图层目录数据
                String geoServerPath = myGeoServerConfig.getGeoServerPath();
                String dataPath = geoServerPath + File.separator + geoserverinfo.getGeoserverlayers();
                //删除文件
                deleteDirectory(dataPath);

                //删除minio
                String pngMinioId = uavSendData.getPngMinioId();
                String objectName = "";
                String prefix = properties.getVisitPoint() + "/" + properties.getBucketName() + "/";
                if (pngMinioId.startsWith(prefix)) {
                    //保存的地址
                    objectName = pngMinioId.substring(prefix.length());
                } else {
                    objectName = pngMinioId;
                }
                try {
                    minioServer.deleteFile(objectName);
                }catch (Exception e) {
                    log.warn("删除Minio文件"+objectName+"失败");
                }

                //删除Geoserver中的数据
                geoserverinfoService.removeById(uavSendData.getGeoserverInfoId());

                //删除数据库中数据
                uavSendDataService.removeById(uavSendData.getId());

                delenum ++;
            }
            //删除Minioo数据
            String minioId = uavData.getMinioId();
            String objectName = "";
            String prefix = properties.getVisitPoint() + "/" + properties.getBucketName() + "/";
            if (minioId.startsWith(prefix)) {
                //保存的地址
                objectName = minioId.substring(prefix.length());
            } else {
                objectName = minioId;
            }
            minioServer.deleteFile(objectName);
            //删除数据库中uavData数据
            uavDataService.removeById(uavData.getId());
        }
        return delenum;
    }

    private void deleteDirectory(String file) {
        deleteDirectory(new File(file));
    }
    /**
     * 递归删除目录及其内容
     *
     * @param file 要删除的目录或文件
     */
    private void deleteDirectory(File file) {
        if (file.isDirectory()) {
            // 如果是目录，则获取其所有子项
            File[] children = file.listFiles();
            if (children != null) {
                for (File child : children) {
                    // 递归删除子目录或文件
                    deleteDirectory(child);
                }
            }
        }
        // 删除文件或空目录
        if (file.delete()) {
            log.debug("Deleted file/directory: " + file.getAbsolutePath());
        } else {
            log.warn("Failed to delete file/directory: " + file.getAbsolutePath());
        }
    }

    public static void main(String[] args) {
        // 指定文件夹路径
        String folderPath0 = "D:\\Users\\田俊彪\\毕业\\railway\\数据\\2023-2024川藏4条河流水质参数反演结果\\2023-2024川藏4条河流水质参数反演结果\\汇总";
        renameFilesInFolder(folderPath0);
//        batchDealTif2GeoserverAndMinioAndDB(folderPath0);
//        String folderPath1 = "D:/Users/田俊彪/毕业/railway/数据/2022-2023川藏4条河流水质参数反演结果/2022/summer";
//        renameFilesInFolder(folderPath1);
//        String folderPath2 = "D:/Users/田俊彪/毕业/railway/数据/2022-2023川藏4条河流水质参数反演结果/2022/autumn";
//        renameFilesInFolder(folderPath2);
//        String folderPath3 = "D:/Users/田俊彪/毕业/railway/数据/2022-2023川藏4条河流水质参数反演结果/2022/winter";
//        renameFilesInFolder(folderPath3);
//        String folderPath4 = "D:/Users/田俊彪/毕业/railway/数据/2022-2023川藏4条河流水质参数反演结果/2023/spring";
//        renameFilesInFolder(folderPath4);
//        String folderPath5 = "D:/Users/田俊彪/毕业/railway/数据/2022-2023川藏4条河流水质参数反演结果/2023/summer";
//        renameFilesInFolder(folderPath5);
//        String folderPath6 = "D:/Users/田俊彪/毕业/railway/数据/2022-2023川藏4条河流水质参数反演结果/2023/autumn";
//        renameFilesInFolder(folderPath6);

    }

    /**
     * 重命名文件夹内所有文件
     *
     * @param folderPath 文件夹路径
     */
    public static void renameFilesInFolder(String folderPath) {
        String prefix = "SAT_RTD_";
        // 创建File对象，表示文件夹
        File folder = new File(folderPath);
        // 获取文件夹内的所有文件和子文件夹
        File[] files = folder.listFiles();

        if (files != null) {
            // 遍历文件夹内的所有文件和子文件夹
            for (File file : files) {
                // 如果是文件，则进行重命名操作
                if (file.isFile()) {
                    // 获取原始文件名
                    String originalName = file.getName();
                    String[] split = originalName.split("_");
                    String newName = null;
                    String type = null;
                    if (split[0].equals("CHLA")) {
                        type = "CPC";
                    } else if (split[0].equals("TLI")) {
                        type = "NTS";
                    } else if (split[0].equals("TSS")) {
                        type = "SPM";
                    } else if (split[0].equals("Zsd")) {
                        type = "TSP";
                    } else if (split[0].equals("NTU")) {
                        type = "TBD";
                    }
                    if (split[2].equals("jinshajiang.tif")) {
                        // 定义新的文件名规则，这里以添加前缀"new_"为例
                        newName = prefix + "JS_00_" + type + "_" + split[1] + "_" + split[1] + "0000.tif";
                    } else if (split[2].equals("nujiang.tif")) {
                        newName = prefix + "NJ_00_" + type + "_" + split[1] + "_" + split[1] + "0000.tif";
                    } else if (split[2].equals("daduhe.tif")) {
                        newName = prefix + "LD_00_" + type + "_" + split[1] + "_" + split[1] + "0000.tif";
                    } else if (split[2].equals("palongazangbujiang.tif")) {
                        newName = prefix + "PL_00_" + type + "_" + split[1] + "_" + split[1] + "0000.tif";
                    } else {
                        continue;
                    }

                    // 创建新的File对象，表示新文件名的文件
                    File newFile = new File(folderPath + "/" + newName);
                    // 重命名文件
                    boolean success = file.renameTo(newFile);
                    if (success) {
                        System.out.println("成功重命名文件：" + originalName + " -> " + newName);
                    } else {
                        System.out.println("重命名文件失败：" + originalName);
                    }
                }
            }
        } else {
            System.out.println("文件夹不存在或无法访问");
        }
    }
}
